/*
  "Polynomial"
  Library supporting two-dimensional linear fits of an injective non-linear function (e.g. sensor data, polynomial)
  with 16 bit resolution. Support point data is being stored in EEPROM.
  V0.10 / Copyright (C) 2020, T.Rode (DL1DUZ)

  Permission is granted to use, copy, modify, and distribute this software
  and documentation for non-commercial purposes.
*/

//################################################################################################################
//includes
//################################################################################################################

#include <Polynomial.h>

//################################################################################################################
//declarations
//################################################################################################################

//################################################################################################################
//functions
//################################################################################################################

// writes the 2 arrays containing the x and y support points to the EEPROM (only changes will be written / subsequent
// calls to this routine will not cause re-writing)
// returns the next (free) address in EEPROM
// "*x" -> pointer to the x-array
// "*y" -> pointer to the y-array
// "size" -> array size
// "address" -> start address in EEPROM

uint16_t PolynomialClass::writeSupportPoints(int16_t *x, int16_t *y, uint16_t size, uint16_t address) {

  return EEPROMDS.dataToEEPROM(EEPROMDS.dataToEEPROM(address, x, 2, size), y, 2, size);

}

//################################################################################################################

// calculate and returns the corresponding y to a given x based on a linear polynomial fit / returns 0 if x
// is out of bound
// "x" -> input value
// "size" -> array size
// "addr_x" -> array start address in EEPROM

int16_t PolynomialClass::getValue(int16_t x, uint16_t size, uint16_t addr_x) {

  int16_t y = 0, xbegin, xend, ybegin, yend;
  uint16_t addr_y = addr_x + (size<<1);

// search for the supporting points enclosing x and if found, calculate y
  for(uint16_t i=0; i<(size<<1)-2; i=i+2) {
    EEPROMDS.dataFromEEPROM(EEPROMDS.dataFromEEPROM(addr_x+i, &xbegin, 2, 1), &xend, 2, 1);

    if((xbegin<=xend && x>=xbegin && x<=xend) || (xbegin>xend && x<=xbegin && x>=xend)) {
      EEPROMDS.dataFromEEPROM(EEPROMDS.dataFromEEPROM(addr_y+i, &ybegin, 2, 1), &yend, 2, 1);

      y = ybegin + (int32_t(yend-ybegin)*int32_t(x-xbegin))/(xend-xbegin);
      break;
    }
  }

  return y;

}

PolynomialClass Poly;
